import {action, makeObservable, observable, runInAction} from "mobx";
import {StockOrderItemModel, StockOrderItemStore} from "../stores/StockOrderItemStore";
import {moduleIsInstalled} from "@rx-suite/stores";
import {errToString, Transport} from "@rx-suite/data";
import {maybeThrowResponse} from "./useOrderEntryStore";
import {FrameCandidate} from "../stores/_FieldConstants";
import {t} from "@lingui/macro";
import {StockOrderModel} from "../stores/StockOrderStore";

class StockOrderEntryStore {
    @observable stockItem: StockOrderItemModel | null = null;
    @observable orderModel: StockOrderModel | null = null;

    @observable showingFrameDialog = false;
    @observable upcLookupFieldValue = "";
    @observable fetchingFrameCandidates = false;
    @observable fetchingFrameCandidatesError = '';
    @observable possibleFrames: FrameCandidate[] = [];

    @observable showingLensDialog = false;
    @observable opcLookupFieldValue = "";
    @observable fetchingLensCandidates = false;
    @observable fetchingLensCandidatesError = '';
    @observable lens = null;

    @observable submittingOrder = false;
    @observable submitErrors = '';

    @observable showingStockItemEditDialog = false;

    @action.bound
    showStockItemEditDialog() {
        this.showingStockItemEditDialog = true;
    }

    @action.bound
    hideStockItemEditDialog() {
        this.showingStockItemEditDialog = false;
    }

    @action.bound
    applyStockItem(stockItem: any) {
        this.stockItem = stockItem
    }

    @action.bound
    handleFrameUPCChange(e: any) {
        this.upcLookupFieldValue = e.target.value;
    }

    @action.bound
    handleLensOPCChange(e: any) {
        this.opcLookupFieldValue = e.target.value;
    }

    @action.bound
    maybeCommitUPCLookupField() {
        if (this.upcLookupFieldValue) {
            this.commitUPCLookupField();
        }
    }

    @action.bound
    maybeCommitOPCLookupField() {
        if (this.opcLookupFieldValue) {
            this.commitOPCLookupField();
        }
    }

    @action.bound
    async commitUPCLookupField() {
        this.resetStockItem();
        await this.findFramesByUPC(this.upcLookupFieldValue)
        runInAction(() => {
            this.upcLookupFieldValue = '';
        })
    }

    @action.bound
    async commitOPCLookupField() {
        this.resetStockItem();
        await this.findLensesByOPC(this.opcLookupFieldValue)
        runInAction(() => {
            this.opcLookupFieldValue = '';
        })
    }

    @action.bound
    async findLensesByOPC(opc: string) {
        if(opc && !this.fetchingLensCandidates) {
            runInAction(() => {
                this.fetchingLensCandidates = true;
                this.fetchingLensCandidatesError = '';
            });
            try {
                const resp = await Transport.fetch(`/api/stock_lenses?id=${encodeURIComponent(opc)}`);
                await maybeThrowResponse(resp)
                const json = await resp.json();
                runInAction(() => {
                    this.lens = json;
                    this.applyLensStock(this.lens);
                })
            } catch (e){
                runInAction(() => {
                    this.fetchingLensCandidatesError = errToString(e);
                })
            } finally {
                runInAction(() => {
                    this.fetchingLensCandidates = false;
                    console.log(JSON.stringify(this.lens))
                })
            }
        } else if (!this.fetchingLensCandidates) {
            runInAction(() => {
                this.lens = null;
                this.fetchingLensCandidatesError = '';
            })
        }
    }

    @action.bound
    async findFramesByUPC(upc: string){
            if (upc && moduleIsInstalled('rx_inventory') && !this.fetchingFrameCandidates){
                runInAction(() => {
                    this.fetchingFrameCandidates = true;
                    this.fetchingFrameCandidatesError = '';
                });
                try {
                    const resp = await Transport.fetch(`/api/rx_inventory_adapter/frames?upc=${encodeURIComponent(upc)}`);
                    await maybeThrowResponse(resp);
                    const json = await resp.json() as FrameCandidate[];
                    runInAction(() => {
                        this.possibleFrames = json;
                        if(this.possibleFrames.length != 0) {
                            this.applyFrameStock(this.possibleFrames[0]);
                        } else {
                            this.fetchingFrameCandidatesError = t`No frame found matching upc: ${upc}`
                        }
                    })
                } catch (e) {
                    runInAction(() => {
                        this.fetchingFrameCandidatesError = errToString(e);
                    })
                } finally {
                    runInAction(() => {
                        this.fetchingFrameCandidates = false;
                    })
                }
            } else if (!this.fetchingFrameCandidates) {
                runInAction(() => {
                    this.possibleFrames = [];
                    this.fetchingFrameCandidatesError = '';
                });
            }
    }

    applyFrameStock(frame: FrameCandidate) {
        if (this.stockItem === null) this.stockItem = StockOrderItemStore.createModel();
        if (frame) {
            this.stockItem.viewModel.item_type = "frame"
            this.stockItem.viewModel.item_identifier = frame.upc
            this.stockItem.viewModel.description = `${frame.upc} ${frame.color} ${frame.a}-${frame.b}-${frame.ed}`
            this.stockItem.viewModel.stock_order_id = this.orderModel?.getId() as string;
        }
    }

    applyLensStock(lens: any) {
        if (this.stockItem === null) this.stockItem = StockOrderItemStore.createModel();
        if (lens) {
            this.stockItem.viewModel.item_type = "lens"
            this.stockItem.viewModel.item_identifier = lens["lt_opc"] || lens["rt_opc"] || lens["stock_number"]
            this.stockItem.viewModel.description = `${lens["lens_code"]} ${lens["manufacturer_desc"]} ${lens["material_desc"]}`
            this.stockItem.viewModel.stock_order_id = this.orderModel?.getId() as string;
        }
    }

    @action.bound
    resetStockItem() {
        this.stockItem = StockOrderItemStore.createModel();
        this.fetchingFrameCandidatesError = '';
        this.fetchingLensCandidatesError = '';
    }

    @action.bound
    showFrameDialog() {
        this.showingFrameDialog = true;
    }

    @action.bound
    hideFrameDialog() {
        this.showingFrameDialog = false;
    }

    @action.bound
    showLensDialog() {
        this.showingLensDialog = true;
    }

    @action.bound
    hideLensDialog() {
        this.showingLensDialog = false;
    }

    @action.bound
    async submitOrder(){
        runInAction(() => {
            this.submitErrors = '';
            this.submittingOrder = true;
        })
        try {
            const response = await Transport.fetch(`/api/stock_orders/${this.orderModel?.getId()}/submit`)
            return await maybeThrowResponse(response)
        } catch (e) {
            runInAction(() => {
                this.submitErrors = errToString(e)
            })
        } finally {
            runInAction(() => {
                this.submittingOrder = false
            })
        }
    }

    constructor() {
        makeObservable(this);
        this.stockItem = StockOrderItemStore.createModel();
    }
}

const stockOrderEntryStore = new StockOrderEntryStore();
// eslint-disable-next-line @typescript-eslint/ban-ts-comment
// @ts-ignore
if (!window.rxstores) {
    // eslint-disable-next-line @typescript-eslint/ban-ts-comment
    // @ts-ignore
    window.rxstores = {};
}
// eslint-disable-next-line @typescript-eslint/ban-ts-comment
// @ts-ignore
if (!window.rxstores.stock_order_entry_store) {
    // eslint-disable-next-line @typescript-eslint/ban-ts-comment
    // @ts-ignore
    window.rxstores.stock_order_entry_store = stockOrderEntryStore;
}

export const useStockOrderStore = () => {
    return stockOrderEntryStore;
}