import * as React from 'react';
import {BreadModel, makeColumn, CheckboxField, makeOptionsColumn} from "@rx-suite/ui";
import {OrderForm} from "./OrderForm";
import {t, Trans} from "@lingui/macro";
import {OrderStore} from "../../stores/OrderStore";
import {DataStore, Model} from '@rx-suite/data';
import {Tab, Tabs} from "@material-ui/core";
import {useCustomerSelectionStore} from "../../hooks/useCustomerSelectionStore";
import {observer} from "mobx-react-lite";
import {Alert} from "@material-ui/lab";
import Typography from "@material-ui/core/Typography";
import {
    FrameStatuses,
    JOB_TYPES,
    mappedFrameStatus,
    mappedJobType
} from "../../stores/_FieldConstants";
import {Filter, RowInfo} from "react-table";
import CSS from "csstype";
import {red, amber} from "@material-ui/core/colors";
import {useMemo} from "react";
import {RxUniverseStore} from "../../stores/RxUniverseStore";
import {useOrderEntryStore} from "../../hooks/useOrderEntryStore";
import {OrdersTabs} from "../OrdersTabs";

const getTrProps = (_: any, rowInfo: RowInfo | undefined) => {
    const props: {style?: CSS.Properties} = {};
    if (rowInfo && rowInfo.original && rowInfo.original.isInvalidJob) {
        props.style = {color: red[500], fontWeight: "bold"};
    }
    if (rowInfo && rowInfo.original && rowInfo.original.hasComment && !rowInfo.original.isInvalidJob) {
        props.style = {color: amber[500], fontWeight: "bold"}
    }
    return props;
};

export const OrdersBreadModel = observer(() => {
    const customerSelectionStore = useCustomerSelectionStore();
    const orderEntryStore = useOrderEntryStore();
    const store = OrderStore as unknown as DataStore<Model>;

    const [tab, setTab] = React.useState<"summary" | "details">("summary");
    const [allAccounts, setAllAccounts] = React.useState(false);
    const [invalidJobs, setInvalidJobs] = React.useState(false);
    const [shippedJobs, setShippedJobs] = React.useState(false);
    const [defaultFiltered, setDefaultFiltered] = React.useState<Array<Filter>>([{id: 'customer_id', value: customerSelectionStore.currentCustomerId}]);

    const releaseLabel = RxUniverseStore.selectedModel.isVA ? t`Last 4` : t`Release Number`;
    const purchaseOrderLabel = RxUniverseStore.selectedModel.isVA ? t`Consult Number` : t`PO Number`;

    const [summaryOrderColumns, setSummaryOrderColumns] = React.useState([]);

    const fetchSummaryOrderColumns = React.useCallback(async() => {
        const columns = await orderEntryStore.getSummaryOrderColumns();

        setSummaryOrderColumns(columns);
    }, [orderEntryStore]);

    React.useEffect(() => {
        fetchSummaryOrderColumns();
    }, [fetchSummaryOrderColumns])

    const detailsColumns = useMemo(() => {
        return [
            makeColumn('order_number', t`Order Number`, "string"),
            makeColumn('customer_num', t`Customer Number`, "string"),
            makeColumn('customer_name', t`Patient Name`, "string"),
            makeColumn('customer_city', t`Customer City`, "string"),
            makeColumn('date_received', t`Date Received`, "date"),
            makeColumn('date_ordered', t`Date Ordered`, "date"),
            makeColumn('cust_job_id', "Customer Job ID", "string"),
            makeColumn('po_number', purchaseOrderLabel, "string"),
            makeColumn('release_number', releaseLabel, "string"),
            makeColumn('hold_at_lab', t`Hold At Lab?`, "boolean"),
            makeColumn('tray', t`Tray`, "string"),
            makeColumn('ship_to_name', t`Ship To Name`, "string"),
            makeColumn('ship_to_address1', t`Ship To Address1`, "string"),
            makeColumn('ship_to_address2', t`Ship To Address2`, "string"),
            makeColumn('ship_to_address3', t`Ship To Address3`, "string"),
            makeColumn('ship_to_address4', t`Ship To Address4`, "string"),
            makeColumn('shipping_method_code', t`Shipping Method`, "string"),
            makeOptionsColumn('job_type', t`Job Type`, JOB_TYPES.map(({ value, label}) => ({label, value})), mappedJobType),
            makeColumn('style_type_name', t`Lens Type`, "string"),
            makeColumn('style_name', t`Lens Style`, "string"),
            makeColumn('style_material_name', t`Lens Material`, "string"),
            makeColumn('lens_color', t`Lens Color`, "string"),
            makeColumn('lens_treat', t`Lens Treat`, "string"),
            makeOptionsColumn('_fstatus', t`Frame Status`, FrameStatuses, mappedFrameStatus),
            makeColumn('fupc', t`Frame UPC`, "string"),
            makeColumn('fram', t`Frame`, "string"),
            makeColumn('_fmupc', t`Frame MUPC`, "string"),
            makeColumn('frame_sku', t`Frame SKU`, "string"),
            makeColumn("trace_id", t`Trace ID`, "string"),
            makeColumn('submitted_by', t`Submitted By`, "string")
        ];
    }, [releaseLabel, purchaseOrderLabel, mappedJobType, mappedFrameStatus])

    const summaryColumns = React.useMemo(() => {
        const selectedOrderColumns = Object.entries(summaryOrderColumns).map(([key, value]) => {
            if (key === "release_number") {
                return makeColumn(key, releaseLabel, "string")
            } else if (key === "_fstatus") {
                return makeOptionsColumn(key, t`Frame Status`, FrameStatuses, mappedFrameStatus)
            } else if (key === "customer_name") {
                return makeColumn(key, t`Patient Name`, "string")
            } else if (key === "job_type") {
                return makeOptionsColumn(key, t`Job Type`, JOB_TYPES.map(({value, label}) => ({label, value})), mappedJobType)
            } else if (key === "hold_at_lab") {
                return makeColumn(key, t`Hold At Lab?`, "boolean")
            } else if (key === "is_shipped") {
                return makeColumn(key, t`Is Shipped`, "boolean")
            } else if (key === "is_valid") {
                return makeColumn(key, t`Is Valid`, "boolean")
            } else if (key === "last_sync_time") {
                return makeColumn(key, t`Last Sync Time`, "date")
            } else if (key === "_fmupc") {
                return makeColumn(key, t`Frame MUPC`, "string")
            } else if (key === "fram") {
                return makeColumn(key, t`Frame`, "string")
            } else if (key === "customer_num") {
                return makeColumn(key, t`Customer Number`, "string")
            } else if (key === "style_material_name") {
                return makeColumn(key, t`Lens Material`, "string")
            } else if (key === "style_name") {
                return makeColumn(key, t`Lens Style`, "string")
            } else if (key === "style_type_name") {
                return makeColumn(key, t`Lens Type`, "string")
            } else if (key === "po_number") {
                return makeColumn(key, purchaseOrderLabel, "string")
            } else if (key === "fupc") {
                return makeColumn(key, t`Frame UPC`, "string")
            } else if (key === "frame_sku") {
                return makeColumn(key, t`Frame SKU`, "string")
            } else if (key === "trace_id") {
                return makeColumn(key, t`Trace ID`, "string")
            } else if (key === "date_received") {
                return makeColumn(key, t`Date Received`, "date")
            } else if (key === "date_ordered") {
                return makeColumn(key, t`Date Ordered`, "date")
            } else if (key === "shipping_method_code") {
                return makeColumn(key, t`Shipping Method`, "string")
            } else {
                return makeColumn(key, value, "string")
            }
        });

        return selectedOrderColumns;

    }, [releaseLabel, mappedFrameStatus, summaryOrderColumns, mappedJobType, purchaseOrderLabel])

    const stockColumns = React.useMemo(() => {
        makeColumn('order_number', t`Order Number`, "string"),
            makeColumn('po_number', t`PO Number`, "string"),
            makeColumn('reference_number', t`Reference Number`, "string"),
            makeColumn('shipping_method_code', t`Shipping Method`, "string"),
            makeColumn('date_completed', t`Date Completed`, "date"),
            makeColumn('date_ordered', t`Date Ordered`, "date")
    }, [])

    const tabs = {
        summary: summaryColumns,
        details: detailsColumns,
        stock: stockColumns
    }

    const columns = tabs[tab];

    const handleAllAccountsChange = React.useCallback(e => {
        setAllAccounts(e.target.checked);
    }, [setAllAccounts]);

    const handleTabChange = React.useCallback((_, t) => {
        setTab(t);
    }, [setTab]);

    const handleInvalidJobChecked = React.useCallback(e => {
        setInvalidJobs(e.target.checked);
    }, [setInvalidJobs])

    const handleShippedJobChecked = React.useCallback(e => {
        setShippedJobs(e.target.checked);
    }, [setShippedJobs])

    React.useEffect(() => {
        const filter = [];

        if(allAccounts) {
            filter.push({id: 'customer_id', value: customerSelectionStore.userCustomers.map(c => c.getId()).join("|")})
        } else {
            filter.push({id: 'customer_id', value: customerSelectionStore.currentCustomerId})
        }

        if(invalidJobs) {
            filter.push({id: 'is_valid', value: !invalidJobs})
        }

        if(shippedJobs) {
            filter.push({id: 'is_shipped', value: true})
        }
        
        setDefaultFiltered(filter);
    }, [shippedJobs, allAccounts, invalidJobs, customerSelectionStore.currentCustomerId])

    return (<>
            {customerSelectionStore.noCustomers && <Alert severity="error">
                <Typography variant={'h6'}>
                    <Trans>This account is not associated with a customer. You will be unable to create orders. Please
                        contact an administrator.</Trans>
                </Typography>
            </Alert>}
            <OrdersTabs />
            <BreadModel store={store} Form={OrderForm} columns={columns}
                        tableProps={{
                            getTrProps: getTrProps,
                            defaultFiltered: defaultFiltered,
                            preTableActions: <>
                                <Tabs value={tab} onChange={handleTabChange}>
                                    <Tab label={t`Summary`} value={'summary'}/>
                                    <Tab label={t`Details`} value={'details'}/>
                                </Tabs>
                                <CheckboxField
                                    label={t`Include All Accounts?`}
                                    onChange={handleAllAccountsChange}
                                    checked={allAccounts}
                                />
                                <CheckboxField
                                    label={t`Show Only Invalid Jobs?`}
                                    onChange={handleInvalidJobChecked}
                                    checked={invalidJobs}
                                />
                                <CheckboxField
                                    label={t`Show Only Shipped Jobs?`}
                                    onChange={handleShippedJobChecked}
                                    checked={shippedJobs}
                                />
                            </>
                        }}/>
        </>
    )
});
