import {ManufacturerUPCField} from "../pages/orders/fields/ManufacturerUPCField";
import {t} from "@lingui/macro";
import {computedFn} from "mobx-utils";
import {FrameLookupButton} from "../pages/orders/frame_inventory_connection/FrameLookupButton";
import {FrameWrapper} from "../pages/orders/frame_inventory_connection/FrameWrapper";
import {AxisField} from "../pages/orders/fields/AxisField";
import {PrismAxisField} from "../pages/orders/fields/PrismAxisField";
import {PrismAmountField} from "../pages/orders/fields/PrismAmountField";
import {FrameNameField} from "../pages/orders/fields/FrameNameField";
import {FrameColorField} from "../pages/orders/fields/FrameColorField";
import {FrameUpcField} from "../pages/orders/fields/FrameUpcField";
import {FrameTraceFields} from "../pages/orders/fields/FrameTraceFields";
import * as React from 'react';
import {FramePreviewButton} from "../pages/orders/frame_inventory_connection/FramePreviewButton";
import {FrameStatusCode} from "./OrderStore";
import { ModalFieldsWrapper } from "../pages/orders/fields/ModalFieldsWrapper";
import { TraceFileButton } from "../components/TraceFileButton";
import { Grid } from "@material-ui/core";

export type Level = {
    label: string,
    buttons?: JSX,
    wrapper?: React.FunctionComponent<any>
};

export const Levels = {
    1: {label: t`Prescription Information`},
    2: {label: t`Prism`, wrapper: ModalFieldsWrapper },
    3: {label: t`Supplemental Information`, wrapper: ModalFieldsWrapper},
    4: {label: t`Blank Overrides`, wrapper: ModalFieldsWrapper},
    5: {label: t`Personalization 1`, wrapper: ModalFieldsWrapper },
    6: {label: t`Personalization 2`, wrapper: ModalFieldsWrapper },
    7: {
        label: t`Frame Measurements`,
        buttons: [
            <Grid item xs={10}>
                <FrameLookupButton key={"lookup"} />
            </Grid>,
            <Grid item xs>
                <FramePreviewButton key={"preview"} />
            </Grid>,
            <Grid item xs={12}>
                <TraceFileButton />
            </Grid>
        ],
        wrapper: FrameWrapper
    },
    8: {label: t`Order personalization`, wrapper: ModalFieldsWrapper },
    9: {label: t`Personalization 3`, wrapper: ModalFieldsWrapper }
} as { [index: string]: Level };

export const levelLabel = computedFn((level: string) => {
    return Levels[level]?.label;
});

export type Field = {
    increment: string,
    initialValue: string,
    level: number,
    mandatory: number,
    maxValue: string,
    minValue: string,
    name: string,
    desc: string,
    options?: {
        code: string,
        desc: string
    }[]
};

export type Fields = {
    PerEye: Field[],
    PerJob: Field[]
}

export type OrderFieldProps = {
    name: string,
    disabled?: boolean,
    label: string,
    options?: { code: string, desc: any }[],
    isSelect?: boolean,
    eye?: string,
    field: Field
};

export const FieldMap: { [index: string]: React.FunctionComponent<OrderFieldProps> } = {
    "_fmupc": ManufacturerUPCField,
    "ax": AxisField,
    "prva2": PrismAxisField,
    "prvm2": PrismAmountField,
    "fram": FrameNameField,
    "fcol": FrameColorField,
    "fupc": FrameUpcField,
    "traceid": FrameTraceFields
};

export type FrameCandidate = {
    id: string,
    upc: string,
    manufacturer_upc: string,
    status: string,

    color: string,
    name: string,
    brand: string,
    collection: string,
    manufacturer: string,
    frame_model: string,
    material: string,
    gender: string,
    part_type: string,
    part_type_code: string,
    edge_type: string,
    edge_type_name: string,

    eye: number | null,
    bridge: number | null,
    a: number | null,
    b: number | null,
    ed: number | null,
    trace: string | null
};

export type FrameTrace = {
    name: string,
    a: number,
    b: number,
    ed: number,
    circ: number,
    dbl: number,
    radii: number[]
}

export type ProcessedOption = { label: string, value: string }
export type UnprocessedOption = string | ProcessedOption
export type RawFrameFindOption = {
    label: string,
    id: string,
    corresponds_to?: string | null,
    options: UnprocessedOption[]
}

export type FrameFindOption = {
    label: string,
    id: string,
    corresponds_to?: string | null,
    options: ProcessedOption[]
}
export type LensUseIndicator = 'USE' | 'HIDE' | 'DISABLE' | 'BALANCE';
export type JobTypeCode = "P" | "R" | "L" | "RB" | "LB" | "F" | "E" |"A" | "I";
export type JobType = { value: JobTypeCode, label: string, right: LensUseIndicator, left: LensUseIndicator };
export const IncludedTraceKeys = ["HBOX", "VBOX", "FED", "CIRC", "DBL", "R"];
export const IncludedFrameKeys = ["_FMUP", "FRAM", "FUPC", "FCOL", "_ETYPE"];
export const ExcludedFieldKeysForEdgeOnlyJobType = ["CLASS", "MBASE", "DIA"];
export const TraceKeyMap = {
    HBOX: "a" ,
    VBOX: "b",
    FED: "ed",
    CIRC: "circ",
    DBL: "dbl"
};

export const JOB_TYPES = [
    {value: "P", label: t`Pair`, right: 'USE', left: 'USE'},
    {value: "R", label: t`Right`, right: 'USE', left: 'HIDE'},
    {value: "L", label: t`Left`, right: 'HIDE', left: 'USE'},
    {value: "RB", label: t`Right Balance`, right: 'BALANCE', left: 'USE'},
    {value: "LB", label: t`Left Balance`, right: 'USE', left: 'BALANCE'},
    {value: "F", label: t`Frame Only`, right: 'HIDE', left: 'HIDE'},
    {value: "E", label: t`Edge Only`, right: 'USE', left: 'USE'},
    {value: "A", label: t`Addon`, right: 'HIDE', left: 'HIDE'},
    {value: "I", label: t`Invalid Job`, right: 'HIDE', left: 'HIDE'},
] as JobType[];

export type FrameStatus = { value: FrameStatusCode, label: string };
export const FrameStatuses = [
    {value: "D", label: t`Uncut no Frame`},
    {value: "U", label: t`Uncut with Frame`},
    {value: "L", label: t`Trace Only`},
    {value: "F", label: t`Frame to Follow`},
    {value: "E", label: t`Frame to Enclosed`},
    {value: "S", label: t`Lab Supplied Frame`},
    {value: "N", label: t`None`}
] as FrameStatus[];

export const mappedFrameStatus = computedFn(function getVal(val: FrameStatusCode | undefined) {
    return val ? FrameStatuses.find( a => a.value == val)?.label as string : "None"
});

export const mappedJobType = computedFn(function getVal(val: string | undefined) {
    return JOB_TYPES.find(a => a.value == val)?.label as string
});

export type FrameFindFilters = { [index: string]: string }

export const sortedOptions = computedFn((options : { label: any, value: any } [] | undefined | {code:string, desc:string}[]) => {
    return options?.sort((a:any, b:any) => {
        if(a.label){
            return (a.label).localeCompare(b.label);
        } else if (a.code){
            return (a.code).localeCompare(b.code);
        }
    }) as [];
});

export const FrameOnlyFields = [
    {
        "level": 7,
        "name": "traceID",
        "desc": "Traced shape to use",
        "mandatory": 0,
        "initialValue": "",
        "minValue": "",
        "maxValue": "",
        "increment": ""
    },
    {
        "level": 7,
        "name": "HBOX",
        "desc": "A box measurement",
        "mandatory": 0,
        "initialValue": "",
        "minValue": "30",
        "maxValue": "80",
        "increment": "0.1"
    },
    {
        "level": 7,
        "name": "VBOX",
        "desc": "B box measurement",
        "mandatory": 0,
        "initialValue": "",
        "minValue": "15",
        "maxValue": "60",
        "increment": "0.1"
    },
    {
        "level": 7,
        "name": "FED",
        "desc": "ED box measurement",
        "mandatory": 0,
        "initialValue": "",
        "minValue": "30",
        "maxValue": "80",
        "increment": "0.1"
    },
    {
        "level": 7,
        "name": "CIRC",
        "desc": "Circumference",
        "mandatory": 0,
        "initialValue": "",
        "minValue": "0",
        "maxValue": "500",
        "increment": "0.1"
    },
    {
        "level": 7,
        "name": "DBL",
        "desc": "Frame DBL",
        "mandatory": 0,
        "initialValue": "",
        "minValue": "7",
        "maxValue": "30",
        "increment": "0.1"
    },
    {
        "level": 7,
        "name": "_ETYPE",
        "desc": "Edge Type Code",
        "mandatory": 0,
        "initialValue": "",
        "minValue": "",
        "maxValue": "",
        "increment": "",
        "options": []
    },
    {
        "level": 7,
        "name": "FRAM",
        "desc": "Frame name",
        "mandatory": 0,
        "initialValue": "",
        "minValue": "",
        "maxValue": "",
        "increment": ""
    },
    {
        "level": 7,
        "name": "FUPC",
        "desc": "Frame UPC",
        "mandatory": 0,
        "initialValue": "",
        "minValue": "",
        "maxValue": "",
        "increment": ""
    },
    {
        "level": 7,
        "name": "FCOL",
        "desc": "Frame color",
        "mandatory": 0,
        "initialValue": "",
        "minValue": "",
        "maxValue": "",
        "increment": ""
    },
    {
        "level": 7,
        "name": "_FMUPC",
        "desc": "Frame manuf UPC",
        "mandatory": 0,
        "initialValue": "",
        "minValue": "",
        "maxValue": "",
        "increment": ""
    }
];

export const PostalRegex = new RegExp('^(\\d{5}(-\\d{4})?|[A-CEGHJ-NPRSTVXY]\\d[A-CEGHJ-NPRSTV-Z] ?\\d[A-CEGHJ-NPRSTV-Z]\\d)$');

export const horizontalOptions = ["In", "in", "I", "i", "Out", "out", "O", "o"]
export const verticalOptions = ["Up", "up", "U", "u", "Down", "down", "D", "d"]
export const prismOptions = [...horizontalOptions, ...verticalOptions]