# == Schema Information
#
# Table name: orders
#
#  id                    :bigint           not null, primary key
#  _fmupc                :string(4000)
#  _fsku                 :string(4000)
#  _fstatus              :string(4000)
#  comments              :string(385)
#  customer_city         :string(4000)
#  customer_name         :string(4000)
#  customer_num          :string(4000)
#  date_completed        :date
#  date_ordered          :date
#  date_received         :date
#  fcol                  :string(4000)
#  fram                  :string(4000)
#  frame_sku             :string(4000)
#  fupc                  :string(4000)
#  has_synced_once       :boolean
#  hold_at_lab           :boolean          default(FALSE), not null
#  is_shipped            :boolean          default(FALSE)
#  is_valid              :boolean          default(TRUE)
#  job_track_description :string(4000)
#  job_track_station     :string(4000)
#  job_type              :string(4000)
#  last_sync_message     :string(4000)
#  last_sync_time        :datetime
#  lens_color            :string(4000)
#  lens_treat            :string(4000)
#  order_number          :string(4000)
#  origin                :string(4000)
#  po_number             :string(4000)
#  raw_data              :text(2147483647)
#  release_number        :string(4000)
#  ship_to_address1      :string(4000)
#  ship_to_address2      :string(4000)
#  ship_to_address3      :string(4000)
#  ship_to_address4      :string(4000)
#  ship_to_code          :string(4000)
#  ship_to_name          :string(4000)
#  ship_to_num           :string(4000)
#  submitted_by          :string(4000)
#  tray                  :string(4000)
#  created_at            :datetime         not null
#  updated_at            :datetime         not null
#  cust_job_id           :string(4000)
#  customer_id           :bigint
#  shipping_method_id    :bigint
#  style_id              :bigint
#  style_material_id     :bigint
#  style_type_id         :bigint
#  trace_id              :string(4000)
#
# Indexes
#
#  index_orders_on_customer_id         (customer_id)
#  index_orders_on_shipping_method_id  (shipping_method_id)
#  index_orders_on_style_id            (style_id)
#  index_orders_on_style_material_id   (style_material_id)
#  index_orders_on_style_type_id       (style_type_id)
#
# Foreign Keys
#
#  fk_rails_...  (customer_id => customers.id)
#  fk_rails_...  (shipping_method_id => shipping_methods.id)
#  fk_rails_...  (style_id => styles.id)
#  fk_rails_...  (style_material_id => style_materials.id)
#  fk_rails_...  (style_type_id => style_types.id)
#
class Order < ApplicationRecord
  include RxModule::TrimmedNumber
  include OrderSync

  belongs_to :style, required: false
  belongs_to :style_material, required: false
  belongs_to :style_type, required: false
  belongs_to :shipping_method, required: false
  belongs_to :customer

  has_many :order_addons, dependent: :destroy
  has_many :addons, through: :order_addons

  has_one_attached :trace_file

  validates_uniqueness_of :po_number, allow_blank: true, allow_nil: true, scope: :customer_id

  def to_json(*args)
    JSON.generate(JSON.parse(super.as_json(*args)).merge(JSON.parse(render_jbuilder_partial)))
  end

  def job_types
    {
      P: "Pair",
      R: "Right",
      L: "Left",
      RB: "Right Balance",
      LB: "Left Balance",
      F: "Frame Only",
      E: "Edge Only",
      A: "Addon",
      I: "Invalid Job"
    }
  end

  def frame_statuses
    {
      D: "Uncut no Frame",
      U: "Uncut with Frame",
      L: "Trace Only",
      F: "Frame to Follow",
      E: "Frame to Enclosed",
      S: "Lab Supplied Frame",
      N: "None"
    }
  end
  
  def to_vca
    raw_data = JSON.parse(self.raw_data)
    vca_data = {
      RXNM: order_number.upcase,
      _ACCN_NAME: customer&.name || "",
      _PONUM: po_number.upcase,
      CLIENT: customer_name,
      _SHPNUM: ship_to_num,
      _SHPNAME: ship_to_name,
      _SHPADD1: ship_to_address1,
      _SHPADD2: ship_to_address2,
      _SHPADD3: ship_to_address3,
      _SHIPADD4: ship_to_address4,
      _TRAY: tray,
      _RELEASENUM: release_number,
      _JOBTYPE: job_types[job_type.to_sym],
      COLR: lens_color,
      _TREAT: lens_treat,
      _FRSTATUSDESC: frame_statuses[_fstatus.to_sym],
      FUPC: fupc,
      FRAM: fram,
      FCOL: fcol,
      _FR_MUPC: _fmupc,
      _FSKU: frame_sku,
      LNAM: style&.style,
      _LSTYLE: style&.style,
      LMATNAME: style_material&.material_code,
      _LCUSTDESC: style&.description,
      _MCUSTDESC: style_material&.material_description,
      _MATERIAL: style_material&.material_code,
      _ORD_DATE: date_ordered,
      _ORD_DATE_YYYYMMDD: date_ordered&.strftime("%Y%m%d"),
      _ORD_DATE_MMDD: date_ordered&.strftime("%m%d"),
      _ORD_DATE_DD: date_ordered&.strftime("%d"),
      _ORD_CDATE: date_completed,
      _ORD_CDATE_YYYYMMDD: date_completed&.strftime("%Y%m%d"),
      _ORD_CDATE_MMDD: date_completed&.strftime("%m%d"),
      _ORD_CDATE_DD: date_completed&.strftime("%d"),
      _ADDON: addons.map(&:code).join(";"),
      _ADDONDESC: addons.map(&:desc).join(";"),
      MESG: comments.split("\n").join(";"),
    }

    trace_keys = %w[TRCFMT R Z ZFMT]

    additional_data = {}
    raw_data.each do |k, v|
      key = k.delete_prefix("rt").delete_prefix("lt")
      additional_data[key] ||= []

      if k.start_with?("rt")
        additional_data[key][0] = v
      elsif k.start_with?("lt")
        additional_data[key][1] = v
      elsif k == "traceID"
        additional_data["_TRACENAME"] = [v]
      elsif !trace_keys.include?(k)
        additional_data[key].push v
      end
    end

    if trace_file.attached?


      trace_data = trace_file.download
      trace_data.lines.each do |line|
        key, value = line.chomp.split("=")

        if trace_keys.include?(key)
          additional_data[key] ||= []
          additional_data[key].push(*value.split(";"))
        end
      end
    end

    additional_data["_EDBOX"] = additional_data["FED"] if additional_data["FED"].present?

    additional_data.each do |k, v|
      vca_data[k] = v.join(";")
    end

    vca_data
  end

  class << self

    private

    def duplicate_order?(po_num, cust_num)
      Order.exists?(po_number: po_num, customer_id: cust_num, date_completed: nil)
    end
  end
end
